
#ifndef __ARMEMUL_H__
#define __ARMEMUL_H__

#include "emumem.h"
#include <cstdio>
#include <queue>
#include <utility>

namespace emulator {

#define DEF_CLASS(a,b) a,
	typedef enum {
#include "inst_class.h"
	} inst_class;

typedef std::pair <unsigned, UInt32 > ns_data;

  class arm_emulator {

	public:

		/* constructor */
		arm_emulator(bool, bool);

		/* destructor */
		~arm_emulator();

		/* run the program */
		void run(UInt64 count);

		/* stop running */
		void stop() {running = false;}

		/* is running? */
		bool is_running() {return running;}

		/* set running */
		void set_running() {running = true;}

		/* execute one instruction */
		void execute(arm_inst_t inst, word_t addr);

		/* debug an application */
		void debug();

		/* stop running */
		void stop_debug() {debuging = false;}

		/* is debuging */
		bool is_debugging() {return debuging;}

		/* stats output */
		void dump_instruction_counters(FILE *);

		/* stats initialization */
		void reset_instruction_counters();

		/* increment counter */
		void increment_counter(inst_class c) {counters[c]++;}

		word_t get_pc() {return my_regs.gpr[15];}

		word_t read_gpr(int index) {
			return my_regs.gpr[index];	
		}

		void write_gpr(int index, word_t val) {
			my_regs.gpr[index==15?my_regs.pcflag=15:index] = val;
		}

		void write_gpr2(int index, word_t val) {
			my_regs.gpr[index] = val;
		}

		bool is_pc_updated() {return my_regs.pcflag!=0;}
		void clear_pc_update() {my_regs.pcflag=0;}

		word_t read_cpsr() {return my_regs.cpsr;}
		word_t read_spsr() {return my_regs.spsr;}

		void write_cpsr(word_t val) {my_regs.cpsr=val;}
		void write_spsr(word_t val) {my_regs.spsr=val;}

		/* fetch an instruction */
		word_t fetch_inst(arm_addr_t addr) {
			return mem.read_word_fast(addr);
		}

		/* set brk point for syscall emulation */
		void set_brk(arm_addr_t addr) {brk_point = addr;}
		void set_mmap_brk(arm_addr_t addr) {mmap_brk_point = addr;}

		void load_program(const char *, int argc, char *argv[], char *env[]);
		void load_fpe(const char *);

		/* counters */
		UInt64 get_icount() {return icount;}
		UInt64 get_ncount() {return ncount;}

		/* reset */
		void reset();
	private:

		/* number of instructions */
		UInt64 icount;

		/* number of nullified instructions */
		UInt64 ncount;
		
		/* number of executed instructions in each class */
		UInt64 counters[INST_TOTAL];

		/* initialize registers */
		void init_registers();

		/* initialize queues */
		void init_queues();

		/* debug routines */
		void debug_trace(int count);
		void debug_go_to(arm_addr_t addr);
		void debug_dump_registers(FILE *stream);
		void debug_disasm(FILE *stream, arm_addr_t addr);
		void debug_dump(FILE *stream, arm_addr_t addr);

		bool running;
		bool debuging;

	public:
		/* registers */
		struct regs_t {
			word_t gpr[16];
			word_t cpsr;
			word_t spsr;
			word_t pcflag;  /*set when pc is updated*/
		} my_regs;

		/* memory */
		memory mem;

		bool verbose;
		bool emulate_syscall;
		target_addr_t brk_point;
		target_addr_t mmap_brk_point;

		int inbuf[2], outbuf[2];
		int inflag, outflag;
		/* added by Xinping  for the distributed simulation */
		unsigned pe_address;
		std::queue<ns_data> *input_queue;
		std::queue<ns_data> output_queue;

		/* emulating fpe */
		bool in_fpe;

		/* dpi carries */
		word_t shifter_carry;
		word_t alu_carry;
  };
} /* namespace */

#endif
