/*************************************************************************
    Copyright (C) 2002,2003 Wei Qin
    See file COPYING for more information.

    This program is free software; you can redistribute it and/or modify    
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
*************************************************************************/

#include <cstdio>
#include <csignal>
#include <cstring>
#include <cstdlib>
#include "armemul.h"
#include "loader.h"
#include "config.h"

using namespace emulator;

static void usage(char *fname)
{
	char *cp;
	if ((cp = rindex(fname, '/'))!=NULL) cp++;
	else cp = fname;
	fprintf(stderr, 
		"**** " PACKAGE " Version " VERSION " (C) 2002,2003 Wei Qin ****\n"
		"usage : %s [-d] [-v] [-f fpe-path] [-h] <file name> <args...>\n"
		"  -d : debuging mode\n"
		"  -v : verbose\n"
		"  -m : maximum number of instructions to simulate\n"
		"  -f : FPE binary file path\n"
		"  -h : print this message and quit\n"
		"  file name : the ARM ELF32 program to simulate\n"
		"  args : arguments to the program\n", cp);
}

static arm_emulator *mltr = NULL;
static void sigint_handler(int signum)
{
	if (mltr->is_running()) {
		mltr->stop();
	}
	else if (mltr->is_debugging()) {
		mltr->stop_debug();
	}
	else exit(0);
}

int main(int argc, char *argv[], char *envp[])
{
	int i;
	char *prog_name = NULL;
	char *fpe_name = NULL;
	bool verbose = false;
	bool debuging = false;
	UInt64 max_inum = (UInt64)-1;
	void (*prev_sigint_handler)(int);

	for(i = 1; i < argc; i++)
	{
		if(strcmp(argv[i], "-d") == 0) debuging = true; else
		if(strcmp(argv[i], "-v") == 0) verbose = true; else
		if(strcmp(argv[i], "-f") == 0) fpe_name = argv[++i]; else
		if(strcmp(argv[i], "-m") == 0) max_inum = ato_uint64(argv[++i]); else
		if(strcmp(argv[i], "-h") == 0) {usage(argv[0]); return 0;} else
		if(!prog_name)
		{
			prog_name = argv[i];
			break;
		}	
	}
	
	if(prog_name)
	{
		/* emulator instance */
		arm_emulator ema(verbose, true);

		mltr = &ema;

		ema.load_fpe(fpe_name);
		ema.load_program(prog_name, argc - i, argv + i, envp);

		prev_sigint_handler = signal(SIGINT, sigint_handler); 

		if(debuging)
			ema.debug();
		else
			ema.run(max_inum);

		if(prev_sigint_handler != SIG_ERR) signal(SIGINT, SIG_DFL);
	}
	else usage(argv[0]);

	return 0;
}
