
//****************************************************************************
//****************************************************************************
//
//
//****************************************************************************
//****************************************************************************


//
// Example using the simple abstract crossbar model
// introduced in section 9.5.
//


#include "ns_arm_bus_node.h"
#include "ns_switch.h"
#include "systemc.h"
#include "flit.h"
#include "address.h"

using namespace simulator;

const int num_nodes = 9;
const int num_links = 2*num_nodes;

const int buffer_size= 128;

ns_arm_bus_node<Flit>  *N[num_nodes];
ns_fifo<Flit> *n_to_n[num_links];
ns_switch<Flit> *my_switch;

/* connect nodes n to its four neighbors, using its source address */
void connect_node_bus(ns_arm_bus_node<Flit>& n1, int address){
  
  
  // output  port-channel connections
    n1.output(*n_to_n[address*2]);
    (*my_switch).inputs(*n_to_n[address*2]);   
  //input  port-channel connections
    n1.input(*n_to_n[address*2 + 1]);
    (*my_switch).outputs(*n_to_n[address*2 + 1]);

}

int sc_main(int argc, char* argv[])
{       
        int i;
        char *prog_name = NULL;
        bool verbose = false;
        bool debuging = false;
        char *fpe_name = NULL;
        // oh, clock
        sc_clock clk("clk", sc_time(10, SC_NS) /*period*/);
               
	char *env[] = {NULL};

        for(i = 1; i < argc; i++)
        {
                if(strcmp(argv[i], "-d") == 0) debuging = true; else
                if(strcmp(argv[i], "-v") == 0) verbose = true; else
                if(strcmp(argv[i], "-f") == 0) fpe_name = argv[++i]; else
                if(!prog_name)
       
         {
                        prog_name = argv[i];
                        break;
                }
        }
        /* initialize the pe array */
        for (int j = 0; j < num_nodes; j ++){
	  char *s;
	  s =(char *)malloc ((size_t)34*sizeof(char));
	  sprintf(s, "%s__%d", "ARMpe", j);
          //fprintf(stderr, "%s \n", s);
          N[j] = new ns_arm_bus_node<Flit>(s, j, prog_name, false, argc - i, argv + i, env); // j is the address
          
	}
        
         for(int j = 0; j < num_links; j++){
          char *s;
          s =(char *) malloc ( (size_t)34*sizeof(char));
          /* create the links */
          sprintf(s, "%s__%d", "ARMlinks", j);
          n_to_n[j] = new ns_fifo<Flit>(s, buffer_size);
	}
   
         /* instantiate the switch */
         my_switch = new ns_switch<Flit>("Switch", num_nodes, 3);
         (*my_switch).clk(clk);
         
         for(int j = 0; j < num_nodes; j++){
          (*N[j]).clk(clk);
          connect_node_bus(*N[j], j);
         }

         // start the simulation without a time limit
         sc_start();


return 0;

}

