
//****************************************************************************
//****************************************************************************
//
//
//****************************************************************************
//****************************************************************************


//
// Example using the simple abstract crossbar model
// introduced in section 9.5.
//


#include "ns_node.h"
#include "systemc.h"
#include "flit.h"
#include "address.h"

using namespace simulator;
const int net_degree = 4;
const int torus_size = 3;
const int num_nodes = torus_size*torus_size;
const int num_links = 4*num_nodes;

const int buffer_size= ns_node<Flit>::buffer_size;

ns_node<Flit>    *N[num_nodes];
ns_fifo<Flit> *n_to_n[num_links];

/* connect nodes n to its four neighbors, using its source address */
void connect_node_torus(ns_node<Flit>& n1, int address, const unsigned torus_size_){
  
  int addr_s, addr_e, addr_n, addr_w;
  addr_s =  get_next_toru_south(address, torus_size_);  
  addr_e =  get_next_toru_east(address, torus_size_); 
  addr_n =  get_next_toru_north(address, torus_size_);  
  addr_w =  get_next_toru_west(address, torus_size_);
  
  /* count number of channels we are going to use 
   * each node use 4 channel for output
   * so from south, east, east, west, clockwise are the sequence 
   * of assigning the 36 channels (size = 3). The channel count is corresponding 
   * to the one who output, so use it address can calculate the number 
   * of channel it uses
   */

  // output  port-channel connections
    n1.outputs(*n_to_n[address*net_degree]);
    n1.outputs(*n_to_n[address*net_degree+1]);
    n1.outputs(*n_to_n[address*net_degree+2]);
    n1.outputs(*n_to_n[address*net_degree+3]);

  //input  port-channel connections
    n1.inputs(*n_to_n[addr_s*net_degree+2]);
    n1.inputs(*n_to_n[addr_e*net_degree+3]);
    n1.inputs(*n_to_n[addr_n*net_degree]);
    n1.inputs(*n_to_n[addr_w*net_degree+1]);
}

int sc_main(int argc, char* argv[])
{       
        int i;
        char *prog_name = NULL;
        bool verbose = false;
        bool debuging = false;
        char *fpe_name = NULL;
        // oh, clock
        sc_clock clk("clk", sc_time(10, SC_NS) /*period*/);
               
	char *env[] = {NULL};

        for(i = 1; i < argc; i++)
        {
                if(strcmp(argv[i], "-d") == 0) debuging = true; else
                if(strcmp(argv[i], "-v") == 0) verbose = true; else
                if(strcmp(argv[i], "-f") == 0) fpe_name = argv[++i]; else
                if(!prog_name)
       
         {
                        prog_name = argv[i];
                        break;
                }
        }
        /* initialize the pe array */
        for (int j = 0; j < num_nodes; j ++){
	  char *s;
	  s =(char *)malloc ((size_t)34*sizeof(char));
	  sprintf(s, "%s__%d", "ARMpe", j);
          //fprintf(stderr, "%s \n", s);
          N[j] = new ns_node<Flit>(s, (net_degree+1), torus_size, j, prog_name, false, argc - i, argv + i, env);
          
	}
        
         for(int j = 0; j < num_links; j++){
          char *s;
          s =(char *) malloc ( (size_t)34*sizeof(char));
          /* create the links */
          sprintf(s, "%s__%d", "ARMlinks", j);
          n_to_n[j] = new ns_fifo<Flit>(s, buffer_size);
	}
   
         for(int j = 0; j < num_nodes; j++){
          (*N[j]).clk(clk);
          connect_node_torus((*N[j]), j, torus_size);
         }

         // start the simulation without a time limit
         sc_start();


return 0;

}

