#ifndef NS_ARM_BUS_NODE_H
#define NS_ARM_BUS_NODE_H

#include <systemc.h>
#include <cstdio>
#include <csignal>
#include <cstring>
#include <cstdlib>
#include "armsim.h"
#include "misc.h"
#include <string>
#include "ns_fifo.h"
#include "flit.h"
#include "address.h"
using namespace simulator;
template <class T> SC_MODULE(ns_arm_bus_node) { 
    sc_port<ns_fifo_in_if<T>, 0> input;
    sc_port<ns_fifo_out_if<T>, 0> output;
    sc_in_clk clk; // the, uh, clock  
    arm_simulator *pe_sim; 
    
    // constructors
    SC_HAS_PROCESS(ns_arm_bus_node);
    
    // constructor w/ name and number of iterations 
    ns_arm_bus_node(sc_module_name NAME, unsigned address,const char *prog_name, bool verbose, const int argc,char *argv[],char *env[]):address_(address){

      pe_sim = new arm_simulator(verbose);
      pe_sim->load_fpe(NULL);
      (*pe_sim).setAddress(address);
      pe_sim->load_program(prog_name, argc, argv, env);
      SC_CTHREAD(process, clk.pos()); 
        } 

     void process() {
       T token;
       bool status, ended, first_out;
       ended = first_out = false;
       while (1){
	 if (input->num_available()){
	   // some input, recv and enqueue it.
           token = input->read();
           int t = token.get_payload();
           int address = token.get_addr();
           pe_sim->ns_enqueue(address,t);
	 }
         status = pe_sim->clock_tick();
         if ((output->num_free()) && (pe_sim->ns_output_queue_size())){
	   //check if we have some output in the queue, if true, send
           emulator::ns_data t = pe_sim->ns_dequeue();
           token.set_addr(address_);
           token.set_dest_addr(t.first);
           token.set_length(LENGTH);
           token.set_type(HEAD);           
           token.set_payload(t.second); 
           output->write(token);
           if (!first_out){
	      cout << "first out of program " << address_ <<" at " << " [t=" 
                 << sc_time_stamp()
                 << "]" << endl;
              first_out = true;
	   }
	 }

         if (!status && !ended){ 
           ended = true; 
           cout << "program of " << address_ <<" ends at " << " [t=" 
                 << sc_time_stamp()
                 << "]" << endl;
           if (address_ == 1) exit(0);
	 }
         wait();
     }
  }

 private:
     unsigned address_;
     
}; 


#endif

