#ifndef NS_CROSSBAR_H
#define NS_CROSSBAR_H

#include "ns_fifo.h"

#define MAX_SIZE 10
#define IN_VALID -1
template <class T> SC_MODULE(ns_crossbar) {

    // ports
    sc_port<ns_fifo_in_if<T>, 0> inputs; // arbitrary # channels
    sc_port<ns_fifo_out_if<T>, 0> outputs; // ditto: outputs
    sc_fifo_in<int> config; // one config port
    sc_in_clk clk; // the, uh, clock
    
    // data
    unsigned last_; // index of last channel granted access
    unsigned matrix_size_;
   
    T stored_token[MAX_SIZE];
    bool has_item[MAX_SIZE];
    static const int DEBUG = 0;
    

    // constructor
    SC_CTOR(ns_crossbar) : last_(-1) {
        
        SC_METHOD(process);
        sensitive << clk.pos();

        matrix_size_ = outputs.size();

    }

   ns_crossbar(sc_module_name NAME, unsigned matrix_size) : 
        sc_module(NAME), matrix_size_(matrix_size) 
        { 
           SC_METHOD(process);
           sensitive << clk.pos();
           assert(matrix_size < MAX_SIZE); // then we run out of ports
           for(unsigned i=0;i<matrix_size_;i++){
         
            has_item[i] = false;
         }
        } 



 void decode_select_token(int token, int *portlist, int count){
  /* original code
  *porti = token / MAX_SIZE;
  *porto = token % MAX_SIZE; 
  */
  int i;
    
    /* for a fully connected crossbar */
    for (i = count -1; i >=0 ; i--){
    portlist[i] = token % MAX_SIZE;
    token = token / MAX_SIZE;
    if ( portlist[i] == MAX_SIZE -1){
      portlist[i] = -1;
    }
  }
 }



    // processes
    void process() {
        int config_token;
        int  portlist[MAX_SIZE];
        // same # channels required
        assert(inputs.size() == outputs.size()); 
        
        /* send out the stored token */
        for (int i=0;i<outputs.size(); i++) {
	  if ( has_item[i] ){
	     outputs[i]->write(stored_token[i]);
	  }
	
        }

        /* check if we got a config token */
        if (config->num_available() ){
          config_token = config->read();
        
        
        for(unsigned i=0;i<matrix_size_;i++){
          portlist[i] = -1;
          has_item[i] = false;
        }
        if (DEBUG == 1){
        cout << name() << " config="  << config_token << " [t=" << 
             sc_time_stamp() << "]" <<endl;
	}
        decode_select_token(config_token, portlist, matrix_size_);
       
        // Iterate over all ports starting with the one that
        // comes after the one that was granted access last time.
        // Transmit data if both data available at the input and
        // space available at the corresponding output.
        for (int i=0;i<inputs.size(); i++) {
         
          if ( portlist[i] != -1){
            int porti =  portlist[i];
            assert((porti<inputs.size()) && (porti >=0));
            if  (inputs[porti]->num_available() && outputs[i]->num_free()){
	        stored_token[i] = inputs[porti]->read();
                stored_token[i].update_route();  
                has_item[i] = true; 
                if (DEBUG == 1){
		cout << "pass " << porti << "-> "<< i <<endl;
                }
	    }else{
	      if (inputs[porti]->num_available() == 0) cout << "cross: no input " << endl;
              if (outputs[i]->num_free() == 0) cout << "cross: no output " << endl; 
	    }
          }  
        }
     }
    }
};

#endif
