#ifndef NS_NODE_H
#define NS_NODE_H

#include "ns_scheduler.h"
#include "ns_arm_node.h"

template <class T> SC_MODULE(ns_node) { 
    
    static const int buffer_size = 32;
    sc_port<ns_fifo_in_if<T>, 0> inputs;
    sc_port<ns_fifo_out_if<T>, 0> outputs;
    sc_in_clk clk; // the, uh, clock  
    ns_scheduler<T> *arb;
    ns_arm_node<T> *pe;

    ns_fifo<T> *src_to_x;
    ns_fifo<T> x_to_snk;
 
    int matrix_size_;
    int address_;
    
    // constructors
    SC_HAS_PROCESS(ns_node);
    
    // constructor w/ name and number of iterations 
     ns_node(sc_module_name NAME, const unsigned matrix_size, const unsigned torus_size, const unsigned address, const char *prog_name, bool verbose, const int argc,char *argv[],char *env[]) : 
       sc_module(NAME), matrix_size_(matrix_size), address_(address)
        { 
          /* connect the source, sink and scheduler together */
          pe = new ns_arm_node<T>("arm_pe",address, prog_name, torus_size, verbose, argc, argv, env);
          arb = new ns_scheduler<T>("arb", matrix_size);
          src_to_x = new ns_fifo<T>("link1", buffer_size);
          
          (*pe).output(*src_to_x);
          (*arb).outputs(x_to_snk);
          (*arb).outputs(outputs);
          
          (*arb).inputs(*src_to_x);
          (*arb).inputs(inputs);
          (*pe).input(x_to_snk);
       
          (*pe).clk(clk);
          (*arb).clk(clk);
          
          
          SC_CTHREAD(process, clk.pos()); 
            
        } 

     void process() { 
       while (1){
    
         wait();
     }
  }

 private:
   
}; 

#endif

