#ifndef NS_SCHEDULER_H
#define NS_SCHEDULER_H

#include "ns_crossbar.h"
#include "ns_router.h"
#include "ns_fifo.h"

template <class T> SC_MODULE(ns_scheduler) { 
    
    sc_port<ns_fifo_in_if<T>, 0> inputs;
    sc_port<ns_fifo_out_if<T>, 0> outputs;
    sc_in_clk clk; // the, uh, clock  
    
    ns_router *r;
    
    ns_crossbar<T>* x;
    
    sc_fifo<int> r_to_x;

    int matrix_size_;

    // constructors
    SC_HAS_PROCESS(ns_scheduler);
    
    // constructor w/ name and number of iterations 
     ns_scheduler(sc_module_name NAME, unsigned matrix_size) : 
       sc_module(NAME), matrix_size_(matrix_size)
        { 
          /* connect the router and crossbar together */
          r = new ns_router("r", matrix_size);
          (*r).select(r_to_x);
         
          x = new ns_crossbar<T>("x", matrix_size);
          
          (*x).inputs(inputs);
          (*x).outputs(outputs);
          (*x).config(r_to_x);
          (*x).clk(clk);
          dest_p = new int [matrix_size];
          SC_CTHREAD(process, clk.pos()); 
            
        } 
     
     void process() { 
       while (1){
         T f;
          
         for (int i=0; i < inputs.size(); i++){
         if(inputs[i]->buf_peek(f)){
           dest_p[i] = f.get_next_route();
             }else{
           dest_p[i] = -1;
             }
         }
         //(*r).print_port_list(dest_p);
         (*r).store_request(dest_p); 
         (*r).process_request();
         
         wait();
     }
  }

 private:
   int *dest_p;
}; 

#endif

