#ifndef NS_SWITCH_H
#define NS_SWITCH_H

#include "ns_fifo.h"
#include "address.h"

#define MAX_SIZE 10
#define IN_VALID -1
template <class T> SC_MODULE(ns_switch) {

    // ports
    sc_port<ns_fifo_in_if<T>, 0> inputs; // arbitrary # channels
    sc_port<ns_fifo_out_if<T>, 0> outputs; // ditto: outputs
    sc_in_clk clk; // the, uh, clock

   
    // data
    unsigned last_; // index of last channel granted access
    unsigned matrix_size_;
    unsigned allowed_conn_; // how many connections are allowed simultaneously    

    T stored_token[MAX_SIZE];
    bool has_item[MAX_SIZE];
    int continue_port[MAX_SIZE];
    bool port_in_used[MAX_SIZE];
    bool port_out_used[MAX_SIZE];
    int contention_count;
    int contention_cycle_count;
    // constructor
    SC_CTOR(ns_switch) : last_(-1) {
        
        SC_METHOD(process);
        sensitive << clk.pos();

        matrix_size_ = outputs.size();

    }

   ns_switch(sc_module_name NAME, unsigned matrix_size, unsigned allowed_connection) : 
     sc_module(NAME), matrix_size_(matrix_size), allowed_conn_(allowed_connection)  
        { 
	   assert(allowed_connection <= matrix_size);
           last_ = -1;
           SC_METHOD(process);
           sensitive << clk.pos();
           contention_count = 0;
           contention_cycle_count = 0;
           assert(matrix_size < MAX_SIZE); // then we run out of ports
           for(unsigned i=0;i<matrix_size_;i++){
	    
            continue_port[i] = -1;
            port_in_used[i] = false;
            port_out_used[i] = false; 
            has_item[i] = false;

         }
        } 


    // processes
    void process() {
        bool contention = false;
        unsigned used_connection = 0;
        // same # channels required
        assert(inputs.size() == outputs.size()); 
        assert ( matrix_size_ <= unsigned (inputs.size()));
        /* send out the stored token */
        for (int i=0;i<outputs.size(); i++) {
	  if ( has_item[i] ){
	     outputs[i]->write(stored_token[i]);
	  }
        }
	/* initialize these guard varialbe */
       for(unsigned i=0;i<matrix_size_;i++){
          has_item[i] = false;
          port_in_used[i] = false;
          port_out_used[i] = false; 
        }
       /* print out the connection table 
        if (DEBUG == 1){
	  cout << endl;
          for(unsigned i=0;i<matrix_size_;i++){
	    cout << " " << i<< "-> " <<continue_port[i]<<endl;
	  }
          
          cout << endl; 
        }

       */
       // first, if there exist any (input, output) pair
       // in the continue set, do them first
       // they shouldn't be more than the allowed_connection pairs 
        for(unsigned i=0;i<matrix_size_;i++){
	  if (continue_port[i] > -1){
	    // porti = "i", porto = continue_port[i]
	    unsigned porti = i;
            int porto = continue_port[i];
            assert(porto <inputs.size());
            assert(used_connection <allowed_conn_);
            
            port_in_used[porti] = true;
            port_out_used[porto] = true;
            used_connection ++;
            has_item[porto] = true;
	    
            if  (inputs[porti]->num_available() && outputs[porto]->num_free()){
	      /* get the token and store it in the outptu port buffer */
              stored_token[porto] = inputs[porti]->read();
              //stored_token[porto].update_route();
              assert( porto == stored_token[porto].get_dest_addr());
              int seq = stored_token[porto].get_type();
              if (seq != TAIL){
		// do nothing
	      }else{
		/* get rid of the continue_port mapping */
                continue_port[i] = -1;
	      }
              
	    }
	  }
	}

       
        // Iterate over all ports starting with the one that
        // comes after the one that was granted access last time.
        // Transmit data if both data available at the input and
        // space available at the corresponding output.
        for (int i=0;i<inputs.size(); i++) {
            int porti = (i+last_+1)% inputs.size();
            int porto = -1;
            T recv_token;
            assert((porti<inputs.size()) && (porti >=0));
            if  (inputs[porti]->num_available() && (!port_in_used[porti])){
	      if(inputs[porti]->buf_peek(recv_token)){
                porto = recv_token.get_dest_addr();
                 assert((porto<inputs.size()) && (porto >=0));
                if (!port_out_used[porto]){
                /* check if we are how many connected we allowed */
                if (used_connection < allowed_conn_){
                  if ( outputs[porto]->num_free()){
              
                    stored_token[porto] = inputs[porti]->read();
                    //stored_token[porto].update_route();  
                    port_in_used[porti] = true;
                    port_out_used[porto] = true;
                    used_connection ++;
                    last_ = porti;
                    has_item[porto] = true; 

                    int seq = stored_token[porto].get_type();
                    int size = stored_token[porto].get_length();
                      if (seq != TAIL && (size > 1)){
		        // add this to the connection list
                        continue_port[porti] = porto;
	              }else{
		        // do nothing
	              }
                  if (DEBUG == 1){
		    cout << "pass " << porti 
                         << "-> "<< porto <<" [t="
                         << sc_time_stamp()<< "]"<<endl;
		   }
		  }
                }else{
		  contention = true;
                  contention_count ++;	
	       }
	      }
	    }
          }  
        }

        /* calculate the real contention now */
        if (contention){
	  contention_cycle_count ++;
	  if ((DEBUG == 0) && (contention_cycle_count % 10000 == 0) ){
	    cout << "contention " << contention_cycle_count <<" "  <<
                 contention_count <<" [t="
		 << sc_time_stamp()<< "]"<<endl; 
	  }
	}
      }
};

#endif
