
/** Defining the class of token manager. */

#ifndef __MANAGER_H__
#define __MANAGER_H__

#include <string>
#include "config.h"
#include "bittypes.h"

namespace osm {

typedef class token Token;
typedef class object Object;

/**
 *	Manager class.
 *	A token manager manages a set of tokens.
 */
typedef class manager {

	public:
		/** Constructor.
		 *  @param n Number of tokens.
		 */
		manager(int n, const std::string& name);

		/** Destructor. */
		virtual ~manager();

		/** reset */
		virtual void reset();

		/** To test if a token is free for allocation.
		 *	@param id ID number of the token.
		 *	@return True if the token is free for Allocation.
		 */
		virtual bool isTokenFreeForAlloc(uint32_t id, Object *obj);

		/** To test if a token is free for test.
		 *	@param id ID number of the token.
		 *	@return True if the token is free for test.
		 */
		virtual bool isTokenFreeForInquire(uint32_t id, Object *obj);

		/** To test if a token is good to be reclaimed, for blocking release
		 *	@token The token to reclaim.
		 *	@return True if the token is free for test.
		 */
		virtual bool isTokenGoodToReclaim(Token *token, Object *obj);

		/** Allocate the specified token to the object.
		 *  @param id ID number of the token to allocate.
		 *	@param obj The object to receive the token.
		 *	@return The allocated token, NULL if it is already allocated.
		 */
		virtual Token *allocateToken(uint32_t id, Object *obj);

		/** Give the token to the object, but the object cannot own the token.
		 *  @param id ID number of the token to allocate.
		 *	@param obj The object to receive the token.
		 *	@return The token, NULL if it is already allocated.
		 */
		virtual Token *inquireToken(uint32_t id, Object *obj);

		/** Reclaim the token with the specified pointer.
		 *  @param token The pointer to the token to reclaim.
		 *  @return True if the token is no longer needed bby the object.
		 */
		virtual bool reclaimToken(Token *token, Object *obj);

		/** Discard the token 
		 *  @param token The pointer to the token to discard.
		 *  @param obj   The object discarding the token.
		 */
		virtual void discardToken(Token *token, Object *obj);

		/** Synchronous update */
		virtual void updateOnClockTick() {}

		/** get the name of the manager */
		const std::string& getName() {return name;}

	protected:

		int nTotal;
		std::string name;
		Token  *tokens;

} Manager;

}

#endif
