
#ifndef __OBJECT_H__
#define __OBJECT_H__

#include "config.h"
#include "bittypes.h"
#include "itinerary.h"

namespace osm {
/** Object class.
 *  An object usually corresponds to an operation
 */
typedef class object {

	public:
		/** Constructor */
		object() {
			this->active = false;
			this->nullified = false;
		}

		/** Destructor */
		~object() {}

		/** initializers */
		void init(uint32_t inst, uint32_t addr) {
			this->inst = inst;
			this->addr = addr;
			this->active = true;
			this->nullified = false;
			this->itn = NULL;
		}

		bool isActive() {return active;}
		void disActive() {active=false;}

		bool isNullified() {return nullified;}
		void nullify() {nullified=true;}

		/** Get dynamic id*/
		uint64_t getDynId() {return dynid;}
		void setDynId(uint64_t id) {this->dynid=id;}
		
		/** Get the instruction*/
		uint32_t getInst() {return inst;}

		/** Get the program counter */
		uint32_t getAddr() {return addr;}

		void activate() {if (itn&& itn->activate(this)) retire();}

		/** To kill the object and to release all tokens */
		void squash() {
			if (itn) itn->cleanup(this);
			active = false;
		}

		/** Graduate by returning all pending resources. */
		void retire() {
			//itn->cleanup(this);
			active = false;
		}

		/** query for itinerary */
		Itinerary *getItinerary() {return itn;}

		/** set itinerary */
		void setItinerary(Itinerary *itn, Token *fetch) {
			this->itn = itn;
			this->itn->tpbuffer[0] = fetch;
		}

		uint32_t *getInfo() {return info;}

	private:
		uint64_t dynid;
		bool active;	/*retired or squashed operation be false*/
		bool nullified;	/*operation with predicate evaluated as false, will
						 still flow through the pipeline, but should not
						 cause data interlocking*/
		uint32_t inst;
		uint32_t addr;

		uint32_t info[4];	/*decode info can be placed here */

		Itinerary *itn;

} Object;

}

#endif
