
#include "object.h"
#include "simple_sched.h"
#include "itinerary.h"
#include "armsim.h"
#include <list>

#ifdef DEBUG
#include <cstdio>
#endif

using std::list;
using namespace osm;

/*clock ticks*/
void scheduler::tick()
{
	list<Object *>::iterator it;

	/*update all managers*/
	(static_cast<simulator::arm_simulator*>(intface))->updateOnClockTick();

#ifdef DEBUG
	for (it=workList.begin(); it!=workList.end();it++) {
		Itinerary *itn = (*it)->getItinerary();
		if (itn) {
		Node *loc = itn->where();
		if (loc) {
			fprintf(stderr, "#object %x:%llx at %s",
				(*it)->getAddr(), (*it)->getDynId(), loc->getName().c_str());
			if (!(*it)->isActive())
				fputs(" squashed", stderr);
			fputs("\n", stderr);
		}
		else 
			fprintf(stderr, "#object %x:%llx retired\n",
				(*it)->getAddr(), (*it)->getDynId());
		}
		else {
			fprintf(stderr, "#object %x:%llx at IF ",
				(*it)->getAddr(), (*it)->getDynId());
			if (!(*it)->isActive())
				fputs(" squashed", stderr);
			fputs("\n", stderr);
		}
	}

	fputs("--clock ticks--\n", stderr);
#endif

	/*activate all objects from the oldest to youngest*/
	for (it=workList.begin(); it!=workList.end();) {
		/*test if squashed*/
		if ((*it)->isActive()) {
			(*it)->activate();
			it++;
		}
		else {
			/*remove the graduated ones or squashed ones*/
			(static_cast<simulator::arm_simulator*>(intface))->retireObject(*it);
			objq.release(*it);
			it = workList.erase(it);
		}
	}

	/*test if the fetch is empty*/
	while ((static_cast<simulator::arm_simulator*>(intface))->canAwake()) {

		/*can fetch now, get a free object*/
		Object *obj = objq.allocate();
		obj->setDynId(dynid);
		workList.push_back(obj);

#ifdef DEBUG
		fprintf(stderr, "Object %x:%llx enqueued.\n", obj->getAddr(), dynid);
#endif
		(static_cast<simulator::arm_simulator*>(intface))->awakeObject(obj);
		obj->activate();
		dynid++;
	}
}

void scheduler::squash(uint64_t dynid)
{
	list<Object *>::reverse_iterator it;
	for (it=workList.rbegin(); it!=workList.rend(); it++) {
		if ((*it)->getDynId()>dynid) {
#ifdef DEBUG
		fprintf(stderr, "Object %x:%llx > %llx squashed\n",
			(*it)->getAddr(), (*it)->getDynId(), dynid);
#endif
			(*it)->squash();
		} else break;
	}
}

