
#ifndef __TOKEN_H__
#define __TOKEN_H__

#include "manager.h"

namespace osm {

typedef class object Object;

/** Token class. 
 *  A token represent a resource.
 */

typedef class token {

	public:

		/** Constructor. */
		token(Manager *m) :
			free(true), tenant(NULL), boss(m) {}

		/** Destructor. */
		~token() {}

		/** getIndex */
		int getIndex() {return index;}

		/** Test if a token is free.
		 *	@return True if the token is free.
		 */
		bool isFree() {return free;}

		/** Get the object holding the token. */
		Object* getObject() {return tenant;}

		/** Release the token */
		void release(Object *obj) {boss->reclaimToken(this, obj);}

		/** Discard the token */
		void discard(Object *obj) {return boss->discardToken(this, obj);}

		/** Set the token as used by an object. Used by Manager only.
		 *	@param obj The object to hold the token.
		 */
		void setUsed(Object *obj) {tenant=obj,free=false;}

		/** Set an object as unused. Used by manager only. */
		void setUnused() {tenant=NULL,free=true;}

		Manager *getBoss() {return boss;}

	protected:
		/** set index */
		void setIndex(int n) {index=n;}

	private:
		bool free;
		Object *tenant;
		Manager *boss;
		int index;	

	friend class manager;

} Token;

}

#endif
