/*************************************************************************
    Copyright (C) 2002,2003 Wei Qin
    See file COPYING for more information.

    This program is free software; you can redistribute it and/or modify    
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
*************************************************************************/

#include "armsim.h"
#include "dpi_itin.h"
#include "derived_itinerary.h"
#include "itinerary_pool.h"

/*data processing instructions*/

/*move*/

using namespace simulator;

Itinerary *itn_mov_1(arm_simulator *sim, arm_inst_t inst)
{
	uint32_t ids[3] = {(uint32_t)-1, (uint32_t)-1, RDFLD};
	Itinerary *itn = sim->itnq->allocate(ITN_DPI1);
	itn->setIds(ids);
	return itn;
}

Itinerary *itn_mov_1s(arm_simulator *sim, arm_inst_t inst)
{
	uint32_t ids[3] = {(uint32_t)-1, (uint32_t)-1, RDFLD};
	Itinerary *itn = sim->itnq->allocate(ITN_DPI1);
	itn->setIds(ids);
	return itn;
}

Itinerary *itn_mov_2(arm_simulator *sim, arm_inst_t inst)
{
	/* mov pc, r? instruction branches in decoding stage, see sa110-timing
	   specification. So change it to a branch instruction.
	   However, one thing that is modeled is that the r? doesnot read
	   from bypass logic. So the current model works faster than real
	   StrongArm when a mov-pc is dependent on the last operation 
	   To model this, one should separate regifile and bypassing,
	   which requires too much change for such a rare case.
	*/
	if (RDFLD!=15 || ((inst>>7) & 0x01f)) {
		uint32_t ids[3] = {RMFLD, (uint32_t)-1, RDFLD};
		Itinerary *itn = sim->itnq->allocate(ITN_DPI1);
		itn->setIds(ids);
		return itn;
	}
	else {

		Itinerary *itn = sim->itnq->allocate(ITN_BRANCH);
		sim->uopBuffer[0] = itn;
		sim->uopBufferSize = 1;

		itn = sim->itnq->allocate(ITN_FILLER);
		uint32_t ids[] = {RMFLD, (uint32_t)-1, 1};
		itn->setIds(ids);
		return itn;
	}
}

Itinerary *itn_mov_2s(arm_simulator *sim, arm_inst_t inst)
{
	uint32_t ids[3] = {RMFLD, (uint32_t)-1, RDFLD};
	Itinerary *itn = sim->itnq->allocate(ITN_DPI1);
	itn->setIds(ids);
	return itn;
}

Itinerary *itn_mov_3(arm_simulator *sim, arm_inst_t inst)
{
	uint32_t ids[4] = {RMFLD, RSFLD, (uint32_t)-1, RDFLD};
	Itinerary *itn = sim->itnq->allocate(ITN_DPI2);
	itn->setIds(ids);
	return itn;
}

Itinerary *itn_mov_3s(arm_simulator *sim, arm_inst_t inst)
{
	uint32_t ids[4] = {RMFLD, RSFLD, (uint32_t)-1, RDFLD};
	Itinerary *itn = sim->itnq->allocate(ITN_DPI2);
	itn->setIds(ids);
	return itn;
}

/*MVN*/
Itinerary *itn_mvn_1(arm_simulator *sim, arm_inst_t inst)
{
	uint32_t ids[3] = {(uint32_t)-1, (uint32_t)-1, RDFLD};
	Itinerary *itn = sim->itnq->allocate(ITN_DPI1);
	itn->setIds(ids);
	return itn;
}

Itinerary *itn_mvn_1s(arm_simulator *sim, arm_inst_t inst)
{
	uint32_t ids[3] = {(uint32_t)-1, (uint32_t)-1, RDFLD};
	Itinerary *itn = sim->itnq->allocate(ITN_DPI1);
	itn->setIds(ids);
	return itn;
}

Itinerary *itn_mvn_2(arm_simulator *sim, arm_inst_t inst)
{
	uint32_t ids[3] = {RMFLD, (uint32_t)-1, RDFLD};
	Itinerary *itn = sim->itnq->allocate(ITN_DPI1);
	itn->setIds(ids);
	return itn;
}

Itinerary *itn_mvn_2s(arm_simulator *sim, arm_inst_t inst)
{
	uint32_t ids[3] = {RMFLD, (uint32_t)-1, RDFLD};
	Itinerary *itn = sim->itnq->allocate(ITN_DPI1);
	itn->setIds(ids);
	return itn;
}

Itinerary *itn_mvn_3(arm_simulator *sim, arm_inst_t inst)
{
	uint32_t ids[4] = {RMFLD, RSFLD, (uint32_t)-1, RDFLD};
	Itinerary *itn = sim->itnq->allocate(ITN_DPI2);
	itn->setIds(ids);
	return itn;
}

Itinerary *itn_mvn_3s(arm_simulator *sim, arm_inst_t inst)
{
	uint32_t ids[4] = {RMFLD, RSFLD, (uint32_t)-1, RDFLD};
	Itinerary *itn = sim->itnq->allocate(ITN_DPI2);
	itn->setIds(ids);
	return itn;
}

/*ADD*/

Itinerary *itn_add_1(arm_simulator *sim, arm_inst_t inst)
{
	uint32_t ids[4] = {RNFLD, (uint32_t)-1, RDFLD};
	Itinerary *itn = sim->itnq->allocate(ITN_DPI1);
	itn->setIds(ids);
	return itn;
}

Itinerary *itn_add_1s(arm_simulator *sim, arm_inst_t inst)
{
	uint32_t ids[4] = {RNFLD, (uint32_t)-1, RDFLD};
	Itinerary *itn = sim->itnq->allocate(ITN_DPI1);
	itn->setIds(ids);
	return itn;
}

Itinerary *itn_add_2(arm_simulator *sim, arm_inst_t inst)
{
	uint32_t ids[4] = {RNFLD, RMFLD, RDFLD};
	Itinerary *itn = sim->itnq->allocate(ITN_DPI1);
	itn->setIds(ids);
	return itn;
}

Itinerary *itn_add_2s(arm_simulator *sim, arm_inst_t inst)
{
	uint32_t ids[4] = {RNFLD, RMFLD, RDFLD};
	Itinerary *itn = sim->itnq->allocate(ITN_DPI1);
	itn->setIds(ids);
	return itn;
}

Itinerary *itn_add_3(arm_simulator *sim, arm_inst_t inst)
{
	uint32_t ids[4] = {RNFLD, RMFLD, RSFLD, RDFLD};
	Itinerary *itn = sim->itnq->allocate(ITN_DPI2);
	itn->setIds(ids);
	return itn;
}

Itinerary *itn_add_3s(arm_simulator *sim, arm_inst_t inst)
{
	uint32_t ids[4] = {RNFLD, RMFLD, RSFLD, RDFLD};
	Itinerary *itn = sim->itnq->allocate(ITN_DPI2);
	itn->setIds(ids);
	return itn;
}

/*ADC*/
Itinerary *itn_adc_1(arm_simulator *sim, arm_inst_t inst)
{
	uint32_t ids[3] = {RNFLD, (uint32_t)-1, RDFLD};
	Itinerary *itn = sim->itnq->allocate(ITN_DPI1);
	itn->setIds(ids);
	return itn;
}

Itinerary *itn_adc_1s(arm_simulator *sim, arm_inst_t inst)
{
	uint32_t ids[3] = {RNFLD, (uint32_t)-1, RDFLD};
	Itinerary *itn = sim->itnq->allocate(ITN_DPI1);
	itn->setIds(ids);
	return itn;
}

Itinerary *itn_adc_2(arm_simulator *sim, arm_inst_t inst)
{
	uint32_t ids[3] = {RNFLD, RMFLD, RDFLD};
	Itinerary *itn = sim->itnq->allocate(ITN_DPI1);
	itn->setIds(ids);
	return itn;
}

Itinerary *itn_adc_2s(arm_simulator *sim, arm_inst_t inst)
{
	uint32_t ids[3] = {RNFLD, RMFLD, RDFLD};
	Itinerary *itn = sim->itnq->allocate(ITN_DPI1);
	itn->setIds(ids);
	return itn;
}

Itinerary *itn_adc_3(arm_simulator *sim, arm_inst_t inst)
{
	uint32_t ids[4] = {RNFLD, RMFLD, RSFLD, RDFLD};
	Itinerary *itn = sim->itnq->allocate(ITN_DPI2);
	itn->setIds(ids);
	return itn;
}

Itinerary *itn_adc_3s(arm_simulator *sim, arm_inst_t inst)
{
	uint32_t ids[4] = {RNFLD, RMFLD, RSFLD, RDFLD};
	Itinerary *itn = sim->itnq->allocate(ITN_DPI2);
	itn->setIds(ids);
	return itn;
}

Itinerary *itn_sub_1(arm_simulator *sim, arm_inst_t inst)
{
	uint32_t ids[3] = {RNFLD, (uint32_t)-1, RDFLD};
	Itinerary *itn = sim->itnq->allocate(ITN_DPI1);
	itn->setIds(ids);
	return itn;
}

Itinerary *itn_sub_1s(arm_simulator *sim, arm_inst_t inst)
{
	uint32_t ids[3] = {RNFLD, (uint32_t)-1, RDFLD};
	Itinerary *itn = sim->itnq->allocate(ITN_DPI1);
	itn->setIds(ids);
	return itn;
}

Itinerary *itn_sub_2(arm_simulator *sim, arm_inst_t inst)
{
	uint32_t ids[3] = {RNFLD, RMFLD, RDFLD};
	Itinerary *itn = sim->itnq->allocate(ITN_DPI1);
	itn->setIds(ids);
	return itn;
}

Itinerary *itn_sub_2s(arm_simulator *sim, arm_inst_t inst)
{
	uint32_t ids[3] = {RNFLD, RMFLD, RDFLD};
	Itinerary *itn = sim->itnq->allocate(ITN_DPI1);
	itn->setIds(ids);
	return itn;
}

Itinerary *itn_sub_3(arm_simulator *sim, arm_inst_t inst)
{
	uint32_t ids[4] = {RNFLD, RMFLD, RSFLD, RDFLD};
	Itinerary *itn = sim->itnq->allocate(ITN_DPI2);
	itn->setIds(ids);
	return itn;
}

Itinerary *itn_sub_3s(arm_simulator *sim, arm_inst_t inst)
{
	uint32_t ids[4] = {RNFLD, RMFLD, RSFLD, RDFLD};
	Itinerary *itn = sim->itnq->allocate(ITN_DPI2);
	itn->setIds(ids);
	return itn;
}

/*SBC*/
Itinerary *itn_sbc_1(arm_simulator *sim, arm_inst_t inst)
{
	uint32_t ids[3] = {RNFLD, (uint32_t)-1, RDFLD};
	Itinerary *itn = sim->itnq->allocate(ITN_DPI1);
	itn->setIds(ids);
	return itn;
}

Itinerary *itn_sbc_1s(arm_simulator *sim, arm_inst_t inst)
{
	uint32_t ids[3] = {RNFLD, (uint32_t)-1, RDFLD};
	Itinerary *itn = sim->itnq->allocate(ITN_DPI1);
	itn->setIds(ids);
	return itn;
}

Itinerary *itn_sbc_2(arm_simulator *sim, arm_inst_t inst)
{
	uint32_t ids[3] = {RNFLD, RMFLD, RDFLD};
	Itinerary *itn = sim->itnq->allocate(ITN_DPI1);
	itn->setIds(ids);
	return itn;
}

Itinerary *itn_sbc_2s(arm_simulator *sim, arm_inst_t inst)
{
	uint32_t ids[3] = {RNFLD, RMFLD, RDFLD};
	Itinerary *itn = sim->itnq->allocate(ITN_DPI1);
	itn->setIds(ids);
	return itn;
}

Itinerary *itn_sbc_3(arm_simulator *sim, arm_inst_t inst)
{
	uint32_t ids[4] = {RNFLD, RMFLD, RSFLD, RDFLD};
	Itinerary *itn = sim->itnq->allocate(ITN_DPI2);
	itn->setIds(ids);
	return itn;
}

Itinerary *itn_sbc_3s(arm_simulator *sim, arm_inst_t inst)
{
	uint32_t ids[4] = {RNFLD, RMFLD, RSFLD, RDFLD};
	Itinerary *itn = sim->itnq->allocate(ITN_DPI2);
	itn->setIds(ids);
	return itn;
}

/*RSB*/

Itinerary *itn_rsb_1(arm_simulator *sim, arm_inst_t inst)
{
	uint32_t ids[3] = {RNFLD, (uint32_t)-1, RDFLD};
	Itinerary *itn = sim->itnq->allocate(ITN_DPI1);
	itn->setIds(ids);
	return itn;
}

Itinerary *itn_rsb_1s(arm_simulator *sim, arm_inst_t inst)
{
	uint32_t ids[3] = {RNFLD, (uint32_t)-1, RDFLD};
	Itinerary *itn = sim->itnq->allocate(ITN_DPI1);
	itn->setIds(ids);
	return itn;
}

Itinerary *itn_rsb_2(arm_simulator *sim, arm_inst_t inst)
{
	uint32_t ids[3] = {RNFLD, RMFLD, RDFLD};
	Itinerary *itn = sim->itnq->allocate(ITN_DPI1);
	itn->setIds(ids);
	return itn;
}

Itinerary *itn_rsb_2s(arm_simulator *sim, arm_inst_t inst)
{
	uint32_t ids[3] = {RNFLD, RMFLD, RDFLD};
	Itinerary *itn = sim->itnq->allocate(ITN_DPI1);
	itn->setIds(ids);
	return itn;
}

Itinerary *itn_rsb_3(arm_simulator *sim, arm_inst_t inst)
{
	uint32_t ids[4] = {RNFLD, RMFLD, RSFLD, RDFLD};
	Itinerary *itn = sim->itnq->allocate(ITN_DPI2);
	itn->setIds(ids);
	return itn;
}

Itinerary *itn_rsb_3s(arm_simulator *sim, arm_inst_t inst)
{
	uint32_t ids[4] = {RNFLD, RMFLD, RSFLD, RDFLD};
	Itinerary *itn = sim->itnq->allocate(ITN_DPI2);
	itn->setIds(ids);
	return itn;
}

/*RSC*/
Itinerary *itn_rsc_1(arm_simulator *sim, arm_inst_t inst)
{
	uint32_t ids[3] = {RNFLD, (uint32_t)-1, RDFLD};
	Itinerary *itn = sim->itnq->allocate(ITN_DPI1);
	itn->setIds(ids);
	return itn;
}

Itinerary *itn_rsc_1s(arm_simulator *sim, arm_inst_t inst)
{
	uint32_t ids[3] = {RNFLD, (uint32_t)-1, RDFLD};
	Itinerary *itn = sim->itnq->allocate(ITN_DPI1);
	itn->setIds(ids);
	return itn;
}

Itinerary *itn_rsc_2(arm_simulator *sim, arm_inst_t inst)
{
	uint32_t ids[3] = {RNFLD, RMFLD, RDFLD};
	Itinerary *itn = sim->itnq->allocate(ITN_DPI1);
	itn->setIds(ids);
	return itn;
}

Itinerary *itn_rsc_2s(arm_simulator *sim, arm_inst_t inst)
{
	uint32_t ids[3] = {RNFLD, RMFLD, RDFLD};
	Itinerary *itn = sim->itnq->allocate(ITN_DPI1);
	itn->setIds(ids);
	return itn;
}

Itinerary *itn_rsc_3(arm_simulator *sim, arm_inst_t inst)
{
	uint32_t ids[4] = {RNFLD, RMFLD, RSFLD, RDFLD};
	Itinerary *itn = sim->itnq->allocate(ITN_DPI2);
	itn->setIds(ids);
	return itn;
}

Itinerary *itn_rsc_3s(arm_simulator *sim, arm_inst_t inst)
{
	uint32_t ids[4] = {RNFLD, RMFLD, RSFLD, RDFLD};
	Itinerary *itn = sim->itnq->allocate(ITN_DPI2);
	itn->setIds(ids);
	return itn;
}

/*AND*/
Itinerary *itn_and_1(arm_simulator *sim, arm_inst_t inst)
{
	uint32_t ids[3] = {RNFLD, (uint32_t)-1, RDFLD};
	Itinerary *itn = sim->itnq->allocate(ITN_DPI1);
	itn->setIds(ids);
	return itn;
}

Itinerary *itn_and_1s(arm_simulator *sim, arm_inst_t inst)
{
	uint32_t ids[3] = {RNFLD, (uint32_t)-1, RDFLD};
	Itinerary *itn = sim->itnq->allocate(ITN_DPI1);
	itn->setIds(ids);
	return itn;
}

Itinerary *itn_and_2(arm_simulator *sim, arm_inst_t inst)
{
	uint32_t ids[3] = {RNFLD, RMFLD, RDFLD};
	Itinerary *itn = sim->itnq->allocate(ITN_DPI1);
	itn->setIds(ids);
	return itn;
}

Itinerary *itn_and_2s(arm_simulator *sim, arm_inst_t inst)
{
	uint32_t ids[3] = {RNFLD, RMFLD, RDFLD};
	Itinerary *itn = sim->itnq->allocate(ITN_DPI1);
	itn->setIds(ids);
	return itn;
}

Itinerary *itn_and_3(arm_simulator *sim, arm_inst_t inst)
{
	uint32_t ids[4] = {RNFLD, RMFLD, RSFLD, RDFLD};
	Itinerary *itn = sim->itnq->allocate(ITN_DPI2);
	itn->setIds(ids);
	return itn;
}

Itinerary *itn_and_3s(arm_simulator *sim, arm_inst_t inst)
{
	uint32_t ids[4] = {RNFLD, RMFLD, RSFLD, RDFLD};
	Itinerary *itn = sim->itnq->allocate(ITN_DPI2);
	itn->setIds(ids);
	return itn;
}

/*EOR*/
Itinerary *itn_eor_1(arm_simulator *sim, arm_inst_t inst)
{
	uint32_t ids[3] = {RNFLD, (uint32_t)-1, RDFLD};
	Itinerary *itn = sim->itnq->allocate(ITN_DPI1);
	itn->setIds(ids);
	return itn;
}

Itinerary *itn_eor_1s(arm_simulator *sim, arm_inst_t inst)
{
	uint32_t ids[3] = {RNFLD, (uint32_t)-1, RDFLD};
	Itinerary *itn = sim->itnq->allocate(ITN_DPI1);
	itn->setIds(ids);
	return itn;
}

Itinerary *itn_eor_2(arm_simulator *sim, arm_inst_t inst)
{
	uint32_t ids[3] = {RNFLD, RMFLD, RDFLD};
	Itinerary *itn = sim->itnq->allocate(ITN_DPI1);
	itn->setIds(ids);
	return itn;
}

Itinerary *itn_eor_2s(arm_simulator *sim, arm_inst_t inst)
{
	uint32_t ids[3] = {RNFLD, RMFLD, RDFLD};
	Itinerary *itn = sim->itnq->allocate(ITN_DPI1);
	itn->setIds(ids);
	return itn;
}

Itinerary *itn_eor_3(arm_simulator *sim, arm_inst_t inst)
{
	uint32_t ids[4] = {RNFLD, RMFLD, RSFLD, RDFLD};
	Itinerary *itn = sim->itnq->allocate(ITN_DPI2);
	itn->setIds(ids);
	return itn;
}

Itinerary *itn_eor_3s(arm_simulator *sim, arm_inst_t inst)
{
	uint32_t ids[4] = {RNFLD, RMFLD, RSFLD, RDFLD};
	Itinerary *itn = sim->itnq->allocate(ITN_DPI2);
	itn->setIds(ids);
	return itn;
}

/*ORR*/
Itinerary *itn_orr_1(arm_simulator *sim, arm_inst_t inst)
{
	uint32_t ids[3] = {RNFLD, (uint32_t)-1, RDFLD};
	Itinerary *itn = sim->itnq->allocate(ITN_DPI1);
	itn->setIds(ids);
	return itn;
}

Itinerary *itn_orr_1s(arm_simulator *sim, arm_inst_t inst)
{
	uint32_t ids[3] = {RNFLD, (uint32_t)-1, RDFLD};
	Itinerary *itn = sim->itnq->allocate(ITN_DPI1);
	itn->setIds(ids);
	return itn;
}

Itinerary *itn_orr_2(arm_simulator *sim, arm_inst_t inst)
{
	uint32_t ids[3] = {RNFLD, RMFLD, RDFLD};
	Itinerary *itn = sim->itnq->allocate(ITN_DPI1);
	itn->setIds(ids);
	return itn;
}

Itinerary *itn_orr_2s(arm_simulator *sim, arm_inst_t inst)
{
	uint32_t ids[3] = {RNFLD, RMFLD, RDFLD};
	Itinerary *itn = sim->itnq->allocate(ITN_DPI1);
	itn->setIds(ids);
	return itn;
}

Itinerary *itn_orr_3(arm_simulator *sim, arm_inst_t inst)
{
	uint32_t ids[4] = {RNFLD, RMFLD, RSFLD, RDFLD};
	Itinerary *itn = sim->itnq->allocate(ITN_DPI2);
	itn->setIds(ids);
	return itn;
}

Itinerary *itn_orr_3s(arm_simulator *sim, arm_inst_t inst)
{
	uint32_t ids[4] = {RNFLD, RMFLD, RSFLD, RDFLD};
	Itinerary *itn = sim->itnq->allocate(ITN_DPI2);
	itn->setIds(ids);
	return itn;
}

/*BIC*/
Itinerary *itn_bic_1(arm_simulator *sim, arm_inst_t inst)
{
	uint32_t ids[3] = {RNFLD, (uint32_t)-1, RDFLD};
	Itinerary *itn = sim->itnq->allocate(ITN_DPI1);
	itn->setIds(ids);
	return itn;
}

Itinerary *itn_bic_1s(arm_simulator *sim, arm_inst_t inst)
{
	uint32_t ids[3] = {RNFLD, (uint32_t)-1, RDFLD};
	Itinerary *itn = sim->itnq->allocate(ITN_DPI1);
	itn->setIds(ids);
	return itn;
}

Itinerary *itn_bic_2(arm_simulator *sim, arm_inst_t inst)
{
	uint32_t ids[3] = {RNFLD, RMFLD, RDFLD};
	Itinerary *itn = sim->itnq->allocate(ITN_DPI1);
	itn->setIds(ids);
	return itn;
}

Itinerary *itn_bic_2s(arm_simulator *sim, arm_inst_t inst)
{
	uint32_t ids[3] = {RNFLD, RMFLD, RDFLD};
	Itinerary *itn = sim->itnq->allocate(ITN_DPI1);
	itn->setIds(ids);
	return itn;
}

Itinerary *itn_bic_3(arm_simulator *sim, arm_inst_t inst)
{
	uint32_t ids[4] = {RNFLD, RMFLD, RSFLD, RDFLD};
	Itinerary *itn = sim->itnq->allocate(ITN_DPI2);
	itn->setIds(ids);
	return itn;
}

Itinerary *itn_bic_3s(arm_simulator *sim, arm_inst_t inst)
{
	uint32_t ids[4] = {RNFLD, RMFLD, RSFLD, RDFLD};
	Itinerary *itn = sim->itnq->allocate(ITN_DPI2);
	itn->setIds(ids);
	return itn;
}

/*CMP*/
Itinerary *itn_cmp_1s(arm_simulator *sim, arm_inst_t inst)
{
	uint32_t ids[3] = {RNFLD, (uint32_t)-1, (uint32_t)-1};
	Itinerary *itn = sim->itnq->allocate(ITN_DPI1);
	itn->setIds(ids);
	return itn;
}

Itinerary *itn_cmp_2s(arm_simulator *sim, arm_inst_t inst)
{
	uint32_t ids[3] = {RNFLD, RMFLD, (uint32_t)-1};
	Itinerary *itn = sim->itnq->allocate(ITN_DPI1);
	itn->setIds(ids);
	return itn;
}

Itinerary *itn_cmp_3s(arm_simulator *sim, arm_inst_t inst)
{
	uint32_t ids[4] = {RNFLD, RMFLD, RSFLD, (uint32_t)-1};
	Itinerary *itn = sim->itnq->allocate(ITN_DPI2);
	itn->setIds(ids);
	return itn;
}

/*CMN*/
Itinerary *itn_cmn_1s(arm_simulator *sim, arm_inst_t inst)
{
	uint32_t ids[3] = {RNFLD, (uint32_t)-1, (uint32_t)-1};
	Itinerary *itn = sim->itnq->allocate(ITN_DPI1);
	itn->setIds(ids);
	return itn;
}

Itinerary *itn_cmn_2s(arm_simulator *sim, arm_inst_t inst)
{
	uint32_t ids[3] = {RNFLD, RMFLD, (uint32_t)-1};
	Itinerary *itn = sim->itnq->allocate(ITN_DPI1);
	itn->setIds(ids);
	return itn;
}

Itinerary *itn_cmn_3s(arm_simulator *sim, arm_inst_t inst)
{
	uint32_t ids[4] = {RNFLD, RMFLD, RSFLD, (uint32_t)-1};
	Itinerary *itn = sim->itnq->allocate(ITN_DPI2);
	itn->setIds(ids);
	return itn;
}

/*TST*/
Itinerary *itn_tst_1s(arm_simulator *sim, arm_inst_t inst)
{
	uint32_t ids[3] = {RNFLD, (uint32_t)-1, (uint32_t)-1};
	Itinerary *itn = sim->itnq->allocate(ITN_DPI1);
	itn->setIds(ids);
	return itn;
}

Itinerary *itn_tst_2s(arm_simulator *sim, arm_inst_t inst)
{
	uint32_t ids[3] = {RNFLD, RMFLD, (uint32_t)-1};
	Itinerary *itn = sim->itnq->allocate(ITN_DPI1);
	itn->setIds(ids);
	return itn;
}

Itinerary *itn_tst_3s(arm_simulator *sim, arm_inst_t inst)
{
	uint32_t ids[4] = {RNFLD, RMFLD, RSFLD, (uint32_t)-1};
	Itinerary *itn = sim->itnq->allocate(ITN_DPI2);
	itn->setIds(ids);
	return itn;
}

/*TEQ*/
Itinerary *itn_teq_1s(arm_simulator *sim, arm_inst_t inst)
{
	uint32_t ids[3] = {RNFLD, (uint32_t)-1, (uint32_t)-1};
	Itinerary *itn = sim->itnq->allocate(ITN_DPI1);
	itn->setIds(ids);
	return itn;
}

Itinerary *itn_teq_2s(arm_simulator *sim, arm_inst_t inst)
{
	uint32_t ids[3] = {RNFLD, RMFLD, (uint32_t)-1};
	Itinerary *itn = sim->itnq->allocate(ITN_DPI1);
	itn->setIds(ids);
	return itn;
}

Itinerary *itn_teq_3s(arm_simulator *sim, arm_inst_t inst)
{
	uint32_t ids[4] = {RNFLD, RMFLD, RSFLD, (uint32_t)-1};
	Itinerary *itn = sim->itnq->allocate(ITN_DPI2);
	itn->setIds(ids);
	return itn;
}

