/*************************************************************************
    Copyright (C) 2002,2003 Wei Qin
    See file COPYING for more information.

    This program is free software; you can redistribute it and/or modify    
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
*************************************************************************/

#ifndef __ITN_QUEUE__
#define __ITN_QUEUE__

#include "derived_itinerary.h"
#include <vector>

namespace simulator {
typedef class itn_pool {

	public:
		itn_pool(std::vector<Itinerary *>& itn_array) : allitns(itn_array) {
			std::vector<Itinerary *>::iterator it;

		   	for (it = itn_array.begin(); it!=itn_array.end(); it++) {
				int type = (*it)->getType();
				freeList[type].push_back(*it);
				nFree[type]++;
			}
		}

		~itn_pool() {
			std::vector<Itinerary *>::iterator it;
		   	for (it = allitns.begin(); it!=allitns.end(); it++)
					delete (*it);
		}

		itn_pool(const itn_pool&);
		itn_pool& operator=(const itn_pool&);

		Itinerary *allocate(int type) {
			Itinerary *itn;
			nFree[type]--;
			assert(nFree[type]>=0);
			itn = (freeList[type])[nFree[type]];
			itn->reset();
			return itn;
		}

		void release(Itinerary *itn) {
			int type = itn->getType();
			(freeList[type])[nFree[type]] = itn;
			nFree[type]++;
		}

	private:
		std::vector<Itinerary *> allitns;
		std::vector<Itinerary *> freeList[ITN_TOTAL];

		int nFree[ITN_TOTAL];

} ItnPool;

}

#endif
