
#include "armsim.h"
#include "misc.h"
#include <cstdio>
#include <cstring>
#include <csignal>

static void usage(char *fname)
{
    char *cp;
    if ((cp = rindex(fname, '/'))!=NULL) cp++;
    else cp = fname;
    fprintf(stderr,
        "**** " PACKAGE " Version " VERSION " (C) 2002,2003 Wei Qin ****\n"
        "usage : %s [-v] [-f fpe-path] [-h] <file name> <args...>\n"
        "  -v : verbose\n"
		"  -m : maximum number of instructions to simulate\n"
        "  -f : FPE binary file path\n"
        "  -h : print this message and quit\n"
        "  file name : the ARM ELF32 program to simulate\n"
        "  args : arguments to the program\n", cp);
}

static emulator::arm_emulator *mltr = NULL;
static void sigint_handler(int signum)
{
	mltr->stop();
}

int main(int argc, char *argv[], char *envp[])
{
	int i;
	char *prog_name = NULL;
	char *fpe_name = NULL;
	bool verbose = false;
	UInt64 max_inum = (UInt64)-1;
	void (*prev_sigint_handler)(int);

	for(i = 1; i < argc; i++)
	{
		if(strcmp(argv[i], "-v") == 0) verbose = true; else
		if(strcmp(argv[i], "-f") == 0) fpe_name = argv[++i]; else
		if(strcmp(argv[i], "-m") == 0) max_inum = ato_uint64(argv[++i]); else
		if(strcmp(argv[i], "-h") == 0) {usage(argv[0]); return 0;} else
		if(!prog_name)
		{
			prog_name = argv[i];
			break;
		}
	}

	if(prog_name)
	{
		simulator::arm_simulator sim(verbose);

		mltr = &(sim.emu);

		sim.load_fpe(fpe_name);
		sim.load_program(prog_name, argc - i, argv + i, envp);

		prev_sigint_handler = signal(SIGINT, sigint_handler);
		for (int i=0; i<100000; i++) sim.clock_tick();
		sim.dump_stats();
	//	sim.run(max_inum);
		if(prev_sigint_handler != SIG_ERR) signal(SIGINT, SIG_DFL);
	}
	else usage(argv[0]);
	return 0;
}

