/*************************************************************************
    Copyright (C) 2002,2003 Wei Qin
    See file COPYING for more information.

    This program is free software; you can redistribute it and/or modify    
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.
*************************************************************************/

#include "derived_itinerary.h"
#include "itinerary_pool.h"
#include "armsim.h"
#include "mult_itin.h"

#undef WRITE_REG
#undef READ_REG

#define WRITE_REG(ind, val) (sim->emu.write_gpr(ind,val))
#define READ_REG(ind) (sim->emu.read_gpr(ind))

using namespace simulator;

static uint32_t mult_cycle(arm_simulator *sim, arm_inst_t inst)
{
	int32_t val = RS;
	if ((val>>11)==-1 || (val>>11)==0) return 1;
	if ((val>>23)==-1 || (val>>23)==0) return 2;
	return 3;
}

/*RN and RD is switched for MLA*/
Itinerary *itn_mla(arm_simulator *sim, arm_inst_t inst)
{
	uint32_t ids[5] = {RMFLD, RSFLD, RDFLD, RNFLD, mult_cycle(sim, inst)};
	Itinerary *itn = sim->itnq->allocate(ITN_MUL);
	itn->setIds(ids);
	return itn;
}

Itinerary *itn_mla_s(arm_simulator *sim, arm_inst_t inst)
{
	uint32_t ids[5] = {RMFLD, RSFLD, RDFLD, RNFLD, 3};
	Itinerary *itn = sim->itnq->allocate(ITN_MUL);
	itn->setIds(ids);
	return itn;
}

/*RN is RD actually for MUL*/
Itinerary *itn_mul(arm_simulator *sim, arm_inst_t inst)
{
	uint32_t ids[5] = {RMFLD, RSFLD, (uint32_t)-1, RNFLD, mult_cycle(sim, inst)};
	Itinerary *itn = sim->itnq->allocate(ITN_MUL);
	itn->setIds(ids);
	return itn;
}

Itinerary *itn_mul_s(arm_simulator *sim, arm_inst_t inst)
{
	uint32_t ids[5] = {RMFLD, RSFLD, (uint32_t)-1, RNFLD, 3};
	Itinerary *itn = sim->itnq->allocate(ITN_MUL);
	itn->setIds(ids);
	return itn;
}

/*RN for high, RD for low*/
Itinerary *itn_smull(arm_simulator *sim, arm_inst_t inst)
{
	uint32_t ids[7] = {RMFLD, RSFLD, (uint32_t)-1, (uint32_t)-1,
		RDFLD, RNFLD, mult_cycle(sim, inst)+1};
	Itinerary *itn = sim->itnq->allocate(ITN_MULL);
	itn->setIds(ids);
	return itn;
}

Itinerary *itn_smull_s(arm_simulator *sim, arm_inst_t inst)
{
	uint32_t ids[7] = {RMFLD, RSFLD, (uint32_t)-1, (uint32_t)-1,
		RDFLD, RNFLD, 3+1};
	Itinerary *itn = sim->itnq->allocate(ITN_MULL);
	itn->setIds(ids);
	return itn;
}

/*RN for high, RD for low*/
Itinerary *itn_smlal(arm_simulator *sim, arm_inst_t inst)
{
	uint32_t ids[7] = {RMFLD, RSFLD, RDFLD, RNFLD, RDFLD, RNFLD,
		mult_cycle(sim, inst)+1};
	Itinerary *itn = sim->itnq->allocate(ITN_MULL);
	itn->setIds(ids);
	return itn;
}

Itinerary *itn_smlal_s(arm_simulator *sim, arm_inst_t inst)
{
	uint32_t ids[7] = {RMFLD, RSFLD, RDFLD, RNFLD, RDFLD, RNFLD, 3+1};
	Itinerary *itn = sim->itnq->allocate(ITN_MULL);
	itn->setIds(ids);
	return itn;
}

Itinerary *itn_umull(arm_simulator *sim, arm_inst_t inst)
{
	uint32_t ids[7] = {RMFLD, RSFLD, RDFLD, RNFLD, RDFLD, RNFLD,
		 mult_cycle(sim, inst)+1};
	Itinerary *itn = sim->itnq->allocate(ITN_MULL);
	itn->setIds(ids);
	return itn;
}

Itinerary *itn_umull_s(arm_simulator *sim, arm_inst_t inst)
{
	uint32_t ids[7] = {RMFLD, RSFLD, RDFLD, RNFLD, RDFLD, RNFLD, 3+1};
	Itinerary *itn = sim->itnq->allocate(ITN_MULL);
	itn->setIds(ids);
	return itn;
}

Itinerary *itn_umlal(arm_simulator *sim, arm_inst_t inst)
{
	uint32_t ids[7] = {RMFLD, RSFLD, RDFLD, RNFLD, RDFLD, RNFLD,
		mult_cycle(sim, inst)+1};
	Itinerary *itn = sim->itnq->allocate(ITN_MULL);
	itn->setIds(ids);
	return itn;
}

Itinerary *itn_umlal_s(arm_simulator *sim, arm_inst_t inst)
{
	uint32_t ids[7] = {RMFLD, RSFLD, RDFLD, RNFLD, RDFLD, RNFLD, 3+1};
	Itinerary *itn = sim->itnq->allocate(ITN_MULL);
	itn->setIds(ids);
	return itn;
}

